# Copyright 2021, Guillermo Adrián Molina
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

__version__ = "3.4.0"
AUTHOR = "Guillermo Adrián Molina"
AUTHOR_EMAIL = "guillermoadrianmolina@hotmail.com"
NAME = "zfs-clone-manager"
PACKAGE_URL = "http://www.github.com/guillermomolina/zfs-clone-manager"
KEYWORDS = "zfs"
DESCRIPTION = "Tool to manage ZFS clones with history metadata"
LICENSE = "LICENSE"
